#include <gtk/gtk.h>
#include <math.h>
#include "gfx/mask.xpm"
#include "gopod.h"

#define GFX_GO "gfx/go.png"
#define GFX_GO_ "gfx/go_.png"
#define GFX_NOTDETECTED "gfx/notdetected.png"
#define GFX_CAPPED "gfx/capped.png"
#define GFX_UNCAPPED "gfx/uncapped.png"

#define SQUARE(x) ((x)*(x))

GtkWidget *pWindow;
GtkWidget *pHBox;
GtkWidget *pEventButton;
GtkWidget *pButton;
GdkPixbuf *iButton;
GdkPixbuf *iButtonGo;
GtkWidget *pEventImage;
GtkWidget *pImage;
GdkPixbuf *iImageNotDetected;
GdkPixbuf *iImageCapped;
GdkPixbuf *iImageUncapped;
GdkBitmap *mask;
GtkStyle *style;
GError *error;

int is_moving = 0;
int move = 0;
int old_x = 0;
int old_y = 0;
int status = NOIPOD;

static gboolean is_in_circle (int rayon, int centre_x, int centre_y, int x, int y)
{
   int dist;
   dist = sqrt (SQUARE(x - centre_x) + SQUARE(y - centre_y));
   if (dist <= rayon)
      return (TRUE);
   return (FALSE);
}

void update_window (void)
{
   if (status == CAPPED)
      gtk_image_set_from_pixbuf (GTK_IMAGE(pImage), iImageCapped);
      
   if (status == UNCAPPED)
      gtk_image_set_from_pixbuf (GTK_IMAGE(pImage), iImageUncapped);

   if (status == NOIPOD)
      gtk_image_set_from_pixbuf (GTK_IMAGE(pImage), iImageNotDetected);
}

static gboolean event_callback(GtkWidget  *event_box, GdkEventButton *event, gpointer data)
{
   int diff_x;
   int diff_y;
   int x;
   int y;

   is_moving = 1;

   if (!old_x && !old_y)
   {
      old_x = (int)event->x_root;
      old_y = (int)event->y_root;
      return (TRUE);
   }

   diff_x = (int)event->x_root - old_x;
   diff_y = (int)event->y_root - old_y;
   
   gtk_window_get_position(GTK_WINDOW(pWindow), &x, &y);
   gtk_window_move(GTK_WINDOW(pWindow), x + diff_x, y + diff_y);
   
   old_x = (int)event->x_root;
   old_y = (int)event->y_root;
   
   return (TRUE);
}

static gboolean button_press_callback(GtkWidget *event_box, GdkEventButton *event, gpointer data)
{
   if (is_in_circle (6, 199, 53, (int)event->x, (int)event->y))
      return (TRUE);
      
   move = g_signal_connect(G_OBJECT(pWindow), "motion_notify_event", G_CALLBACK(event_callback), pImage);
      return (TRUE);
}

static gboolean button_release_callback (GtkWidget *event_box, GdkEventButton *event, gpointer data)
{
   if (is_in_circle (6, 199, 53, (int)event->x, (int)event->y))
   {
      gtk_main_quit();
      return (TRUE);
   }
   
   if (is_moving)
   {
      gtk_signal_disconnect (G_OBJECT(pWindow), move);
      old_x = 0;
      old_y = 0;
      is_moving = 0;
      return (TRUE);
   }
}

static gboolean enter_notify_callback(GtkWidget *event_box, GdkEventButton *event, gpointer data)
{
   if (is_in_circle (24, 40, 53, (int)event->x, (int)event->y))
   {
      gtk_image_set_from_pixbuf (GTK_IMAGE (pButton), iButtonGo);
      return (TRUE);
   }
   return (FALSE);
}

static gboolean leave_notify_callback(GtkWidget *event_box, GdkEventButton *event, gpointer data)
{
   if (is_in_circle (24, 40, 53, (int)event->x, (int)event->y))
   {
      if (status == NOIPOD)
      {
         if (detectIpod ())
            status = isCapped ();
         else
            status = NOIPOD;
      }

      if (status == CAPPED)
      {
         unCap ();
         status = isCapped (); 
      }
      
      else if (status == UNCAPPED)
      {
         reCap ();
         status = isCapped ();  
      }
    
      update_window();
   }
   
   gtk_image_set_from_pixbuf(GTK_IMAGE(pButton), iButton);
   return (FALSE);
}

void initGUI (void)
{
   pWindow = gtk_window_new(GTK_WINDOW_POPUP);
   gtk_window_set_position(GTK_WINDOW(pWindow), GTK_WIN_POS_CENTER);
   gtk_window_set_default_size(GTK_WINDOW(pWindow), 281, 103);
   gtk_window_set_title(GTK_WINDOW(pWindow), "goPod");
   gtk_widget_show(pWindow);

   style = gtk_widget_get_default_style();
   gdk_pixmap_create_from_xpm_d(pWindow->window, &mask, &style->bg[GTK_STATE_NORMAL], mask_xpm);
   gtk_widget_shape_combine_mask(pWindow, mask, 0, 0);

   g_signal_connect(G_OBJECT(pWindow), "destroy", G_CALLBACK(gtk_main_quit), NULL);
   pHBox = gtk_hbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(pWindow), pHBox);

   iButton = gdk_pixbuf_new_from_file(GFX_GO, &error);
   iButtonGo = gdk_pixbuf_new_from_file(GFX_GO_, &error);
   pButton = gtk_image_new_from_pixbuf(iButton);

   pEventButton = gtk_event_box_new();
   gtk_container_add(GTK_CONTAINER(pEventButton), pButton);
   gtk_box_pack_start(GTK_BOX(pHBox), pEventButton, FALSE, FALSE, 0);

   iImageNotDetected = gdk_pixbuf_new_from_file(GFX_NOTDETECTED, &error);
   iImageCapped = gdk_pixbuf_new_from_file(GFX_CAPPED, &error);
   iImageUncapped = gdk_pixbuf_new_from_file(GFX_UNCAPPED, &error);
   pImage = gtk_image_new_from_pixbuf(iImageNotDetected);
   
   pEventImage = gtk_event_box_new();
   gtk_container_add(GTK_CONTAINER(pEventImage), pImage);
   gtk_box_pack_start(GTK_BOX(pHBox), pEventImage, FALSE, FALSE, 0);
   
   g_signal_connect(G_OBJECT(pWindow), "button_release_event", G_CALLBACK(button_release_callback), pWindow);
   g_signal_connect(G_OBJECT(pWindow), "button_press_event", G_CALLBACK(button_press_callback), pWindow);
   g_signal_connect(G_OBJECT(pEventButton), "button_press_event", G_CALLBACK(enter_notify_callback), pButton);
   g_signal_connect(G_OBJECT(pEventButton), "button_release_event", G_CALLBACK(leave_notify_callback), pButton);
   gtk_widget_show_all(pWindow);
   
   update_window();
   gtk_main();    
   
}


int main (int argc,char **argv)
{
   gtk_init(&argc, &argv);

   if (detectIpod())
      status = isCapped();

   initGUI ();

   return 0;
}
